#include "WiFiClientSecure.h"
#include "fioth.h"

DataLog::DataLog(String ssid, String password, String API)
{
  doinitWifi = 1;
  this->ssid = ssid;
  this->password = password;
  this->_API = API;
  logURL = "https://fioth.vercel.app/api/log";
}

DataLog::DataLog(String API)
{
  this->_API = API;
  doinitWifi = 0;
  logURL = "https://fioth.vercel.app/api/log";
}
void DataLog::init(int muteMessage)
{

  Serial.println(rootCACertificate);
  this->muteMessage = muteMessage ? 1 : 0;

  if (doinitWifi)
  {
    WiFi.begin(ssid, password);
    while (WiFi.status() != WL_CONNECTED)
    {
      delay(500);
      Serial.print(".");
    }
    Serial.println("");
    Serial.print("Connected to WiFi network with IP Address: ");
    Serial.println(WiFi.localIP());
  }
  delay_p = 60000; // 1 log request in 5 mins
  lastTime = millis();
  isMessageFinished = 0;
  jsonMessage = R"("{\"sensor_api\":\")" + _API + R"(\",\"data\":[)";
  // if (task)
  // {
  //     xTaskCreate(
  //         task1,
  //         "task1",
  //         2000,
  //         NULL,
  //         1,
  //         t1);
  // }
}

void DataLog::AddData(String field, String value, String time)
{
  jsonMessage += R"({\"field\":\")";
  jsonMessage += field;

  jsonMessage += R"(\",\"value\":\",)";
  jsonMessage += value;
  jsonMessage += R"(\",\"client_time\":\")";
  jsonMessage += time;
  jsonMessage += R"(\"},)";
  if (!muteMessage)
    Serial.println("data added");

  isData = 1;
}

void DataLog::sendMessage()
{

  if (millis() - lastTime > delay_p && isData)
  {

    if (!isMessageFinished)
    {
      jsonMessage[jsonMessage.length() - 1] = ' ';

      jsonMessage += R"(]}")";
      isMessageFinished = 1;
    }
    if (!muteMessage)
      Serial.println(jsonMessage);
    if (send() == 200)
    {
      lastTime = millis();
      isMessageFinished = 0;
      if (!muteMessage)
        Serial.println("Data successfully logged");
    }
    else
    {
      lastTime = millis() - 30000;
      Serial.print("Log error: ");
      Serial.println(responseCode);
    }
  }
}
int DataLog::send()
{
  if (!isData)
    return 0;

  // WiFiClient client;
  WiFiClientSecure *client = new WiFiClientSecure;
  // if(client){
    // client->setCertificate(rootCACertificate);
    client->setInsecure();
  HTTPClient http;
  http.begin(*client, logURL);
  http.addHeader("Content-Type", "application/json");
  int httpResponseCode = http.POST(jsonMessage);
  if (httpResponseCode == 200)
  {
    isMessageFinished = 0;
    isData = 0;
    lastTime = millis();
  }
    else {
   Serial.printf("[HTTPS] GET... failed, error: %s\n", http.errorToString(httpResponseCode).c_str());
}
  String payload = http.getString();
      Serial.println(payload);
  responseCode = httpResponseCode;
  http.end();
  return httpResponseCode;
 
}
void DataLog::clearUnsendMessages()
{
  jsonMessage = R"("{\"sensor_api\":\")" + _API + R"(\",\"data\":[)";
  isData = 0;
}


